from lyzr.data_analyzr.analysis_handler.sql import TxttoSQLFactory
from lyzr.data_analyzr.analysis_handler.pythonic import PythonicAnalysisFactory
from lyzr.data_analyzr.analysis_handler.plotter import PlotFactory
